/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: bma.c                                                       *
 *                                                                     *
 * Description:                                                        *
 *  Bitmap Array implementation.                                       *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/mm.h>

#include <asm/bitops.h>
#include <asm/semaphore.h>

#include "logging.h"
#include "bma.h"

/* initialize bma */
struct bma_t *bma_init(int size) {

    struct bma_t *bma;
    int rsize;

    bma = (struct bma_t *)kmalloc(sizeof(struct bma_t), GFP_KERNEL);
    if (!bma) {
        Err("Can't allocate bma struct");
        return NULL;
    }

    rsize = size / 8;
    if ((rsize % 8))
        rsize++;

    rsize += sizeof(BMA_DATA_TYPE); /* don't worry about overflow in foreach :) -- ms */
    bma->size = size;
    bma->fpos = 0;
    bma->data = (BMA_DATA_TYPE *)kmalloc(rsize, GFP_KERNEL);
    if (!bma->data) {
        Err("Can't allocate bma data with size %lu", rsize);
        kfree(bma);
        return NULL;
    }

    memset(bma->data, 0, rsize);

    return bma;
}

/* free bma */
void bma_deinit(struct bma_t *bma) {

    if (!bma)
        return;

    kfree(bma->data);
    kfree(bma);
}
