/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: bma.h                                                       *
 *                                                                     *
 * Description:                                                        *
 *  Bitmap Array implementation.                                       *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __COMMON_BMA_H__
#define __COMMAN_BMA_H__

#include <asm/bitops.h>
#include <asm/semaphore.h>

#include "logging.h"

#include "vm_compiler.h"

#define EMPTY_MASK 0x00000000

/* base type */
#define BMA_DATA_TYPE 	    u_int32_t
#define BMA_DATA_TYPE_SIZE  (sizeof(BMA_DATA_TYPE) * 8)

struct bma_t {
    /* array data */
    BMA_DATA_TYPE *data;
    /* size of array */
    int size;
    /* position in foreach */
    int fpos;
};

/* zero fpos for foreach */
#define BMA_START_FOREACH(x) (x->fpos = 0)

/* get foreach position */
#define BMA_GET_FPOS(x) (x->fpos - 1)

/* decls */
struct bma_t *bma_init(int);
void bma_deinit(struct bma_t *);

/* set bit in bma */
static inline void bma_set(struct bma_t *bma, int pos, int val) {

    int offt, posin;
    if (!bma)
        return;

    if (pos > bma->size)
        return;

    offt = (pos / BMA_DATA_TYPE_SIZE);
    posin = pos - (offt * BMA_DATA_TYPE_SIZE);

    if (val) {
        set_bit(posin, (unsigned long *)&bma->data[offt]);
    } else {
        clear_bit(posin, (unsigned long *)&bma->data[offt]);
    }
}

/* test bit in bma */
static inline int bma_test(struct bma_t *bma, int pos) {

    int offt, posin;

    if (!bma)
        return 0;

    if (pos > bma->size)
        return 0;

    offt = (pos / BMA_DATA_TYPE_SIZE);
    posin = pos - (offt * BMA_DATA_TYPE_SIZE);

    return test_bit(posin, (const unsigned long *)&bma->data[offt]);
}

/* foreach bma */
static inline int bma_foreach(struct bma_t *bma) {

    BMA_DATA_TYPE *test;
    int i;

    if (!bma)
        return -1;

    for(;bma->fpos < bma->size; bma->fpos += BMA_DATA_TYPE_SIZE) {
        test = (BMA_DATA_TYPE *)(bma->data + (bma->fpos / BMA_DATA_TYPE_SIZE));
        if (!(*test & EMPTY_MASK)) {
            if ((i = ffs(*test))) {
                bma->fpos = ((bma->fpos / BMA_DATA_TYPE_SIZE) * BMA_DATA_TYPE_SIZE) + i; /* ffs return current pos, fix real offset -- ms */
                return 1;
            }
        }
    }
    return -1;
}

#endif /* __COMMON_BMA_H__ */
