/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: idata.c                                                     *
 *                                                                     *
 * Description:                                                        *
 *  Internal driver data routines.                                     *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/mm.h>
#include <linux/vmalloc.h>

#include "vmmain.h"
#include "mm.h"
#include "intgate.h"
#include "logging.h"

inline void iDataFree(struct module_param_t *param) {
    
    vmTimerKill(&param->timer);
    if (param)
        kfree(param);
}

void iDataClose(struct module_param_t *param) {

    if (!param)
        return;

    /* close */
    if (param->iData.PmmData) {
        mmDeinitPMM(param->iData.PmmData, param);
        param->iData.PmmData = NULL;
    }

#ifdef CALL_MONITOR_FROM_GATE
    if (param->iData.MonId != GATE_INV_MON_ID) {
        ig_FreeMonId(param->iData.MonId);
        param->iData.MonId = GATE_INV_MON_ID;
    }
#endif

    param->MonAddr = NULL;
}

struct module_param_t *iDataOpen(void) {

    struct module_param_t *param;
    struct sysinfo si;
    int i;
    
    if (!(param = (struct module_param_t *)kmalloc(sizeof(struct module_param_t), GFP_KERNEL))) {
        Err("vmOpen: can't allocate memory for module param");
        return NULL;
    }

    /* setup param structure */
    param->iData.PmmData = NULL;
    param->iData.pvm = NULL;
    param->iData.pmmShareBuff = NULL;
#ifdef CALL_MONITOR_FROM_GATE
    param->iData.MonId = GATE_INV_MON_ID;
#endif

    param->MonAddr = NULL;
    param->MonSize = 0;

    param->saltPassed = 0;
    
    for (i = 0; i < MONFUNC_COUNT; i++) {
        param->iData.MonitorFuncs[i] = NULL;
    }

    INIT_LIST_HEAD(&param->timer.list);
    param->timer.countdown = 0;
    init_waitqueue_head(&(param->timer.wq));

    /* alloc bma */
    si_meminfo(&si);
    param->iData.bma = bma_init(si.totalram);

    return param;
}
