/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: logging.c                                                   *
 *                                                                     *
 * Description:                                                        *
 *  Logging funstions.                                                 *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/sched.h>
#include "logging.h"
#include "vm_compiler.h"

/* write message to kernel log */
static void logOutput(const char *prefix, const char *file, const int line, const char *fmt, va_list ap) {

    char buff[LOG_BUFFER_SIZE];

    vsnprintf(buff, LOG_BUFFER_SIZE - 1, fmt, ap);

    if (file) {
        printk("%s[%5d] %s (%s:%d)\n", ((prefix) ? prefix : ""), current->pid, buff, file, line);
    } else {
        if (line < 0) { /* monitor lines */
            printk("[%5d] %s", current->pid, buff);
        } else {
            printk("%s[%5d] %s\n", ((prefix) ? prefix : ""), current->pid, buff);
        }
    }
}

/* monitor logging callback */
void STD_INTERCOM logMonitorCallback(char *fmt, ...) {

#ifdef LOGGING_ON
    va_list ap;

    va_start(ap, fmt);
    logOutput(NULL, NULL, -1, fmt, ap);
    va_end(ap);

#endif
}

/* main logging function */
void STDCALL logPreOutput(const int type, const char *file, const int line, char *fmt, ...) {

    va_list ap;

    va_start(ap, fmt);
    switch(type) {
    case LOG_NORMAL:
        logOutput(NULL, NULL, line, fmt, ap);
        va_end(ap);
        break;
    case LOG_WARN:
        logOutput(KERN_WARNING, file, line, fmt, ap);
        va_end(ap);
        break;
    case LOG_ERROR:
        logOutput(KERN_ERR, file, line, fmt, ap);
        va_end(ap);
        break;
    case LOG_DEBUG:
#ifdef DEBUG_MODE
        logOutput(NULL, file, line, fmt, ap);
#endif
        va_end(ap);
        break;
    case LOG_PANIC:
        logOutput(KERN_EMERG, file, line, fmt, ap);
        va_end(ap);
        break;
    }
}
