/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: logging.h                                                   *
 *                                                                     *
 * Description:                                                        *
 *  Logging functions defines and macros for logging                   *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __COMMON_LOGGING_H__
#define __COMMON_LOGGING_H__

#include "vm_compiler.h"

enum {
    LOG_NORMAL,
    LOG_WARN,
    LOG_ERROR,
    LOG_DEBUG,
    LOG_PANIC
};

/*
 * Logging macros. You must call it, don't call log functions directly.
 */

/* Normal log message */
#define Log(x...)   logPreOutput(LOG_NORMAL, __FILE__, __LINE__, x)

/* Warning message */
#define Warn(x...)  logPreOutput(LOG_WARN, __FILE__, __LINE__, x)

/* Error message */
#define Err(x...)   logPreOutput(LOG_ERROR, __FILE__, __LINE__, x)

/* Debug message (show only with #define DEBUG_MODE) */
#define Debug(x...) logPreOutput(LOG_DEBUG, __FILE__, __LINE__, x)

/* Panic message. Note: Don't use if unsure, because it invoke BUG() ! */
#define Panic(x...) { \
    logPreOutput(LOG_PANIC, __FILE__, __LINE__, x); \
    BUG(); }

void STDCALL logPreOutput(const int type, const char *file, const int line, char *fmt, ...);
void STD_INTERCOM logMonitorCallback(char *fmt, ...);

#define LOG_BUFFER_SIZE 1024

#endif /* __COMMON_LOGGING_H__*/
