/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: utils.c                                                     *
 *                                                                     *
 * Description:                                                        *
 *  Various utilities                                                  *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/sched.h>
#include <linux/file.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

#include "logging.h"
#include "utils.h"
#include "md5.h"

#include "binary-md5.h"

/* static variables */
static char base_md5[] = PVS_BINARY_MD5;

/*
 * Magic function, very "impressive" code :) 
 *                              -- ms
 */
int QUICKCALL check_called_proc(void) {

    char nbuff[sizeof(PVS_BINARY_NAME) + 1], cmdline[CMDLINE_LEN];
    struct file *fp, *mfp;
    mm_segment_t rfs;
    loff_t pos = 0;
    int ret = -EINVAL;
    unsigned char md5buff[md5RSIZE], md5str[md5SIZE + 1];

    snprintf(cmdline, CMDLINE_LEN, CMDLINE_PATH, current->pid);

    fp = filp_open(cmdline, O_RDONLY, 0);
    if (IS_ERR(fp)) {
        Err("Can't open %s: %lu", cmdline, PTR_ERR(fp));
        goto err_open;
    }
    
    rfs = get_fs();
    set_fs(KERNEL_DS);
    memset(nbuff, 0, sizeof(nbuff));
    if ((ret = fp->f_op->read(fp, (void *)nbuff, sizeof(nbuff) - 1, &pos)) < 0) {
        Err("Can't read from %s: %d", cmdline, ret);
        goto err_read;
    }

    ret = -EFAULT;
    /*
     * Here some fixes for RH 9.0 where under non root user
     * /proc/<pid>/cmdline is empty
     */
    if ((!strncmp(nbuff, PVS_BINARY_NAME, sizeof(PVS_BINARY_NAME)))
        || !strlen(nbuff)) {
        mfp = filp_open(((!strlen(nbuff)) ? PVS_BINARY_NAME : nbuff), O_RDONLY, 0);
        if (IS_ERR(mfp)) {
            Err("Can't open %s: %lu", nbuff, PTR_ERR(mfp));
            goto err_read;
        }

        if (!md5_stream(mfp, md5buff)) {
            memset(md5str, 0, sizeof(md5str));
            snprintf(md5str, sizeof(md5str),
                    "%08lx%08lx%08lx%08lx",
                    ((u_long *)md5buff)[0],
                    ((u_long *)md5buff)[1],
                    ((u_long *)md5buff)[2],
                    ((u_long *)md5buff)[3]);
            if (!strncmp(md5str, base_md5, md5SIZE)) {
                ret = 0; /* ok, now we're do very big work and everything looks fine */
            } else {
                Log("MD5sum [%s] does not correspond to the currently installed %s %s-%d.%d build", 
                        md5str, TARGET_NAME_DISP, TARGET_VERSION, VER_BUILD, VER_FIXES);
                Log("Try to reinstall %s %s to fix this error.", TARGET_NAME_DISP, TARGET_VERSION);
            }
        } else {
            Log("md5 count error");
        }
        filp_close(mfp, current->files);
    } else {
        Log("Unknown process called vm-main driver, access has been denied.");
    }
    
err_read:
    filp_close(fp, current->files);
    set_fs(rfs);
    
err_open:

    return ret;
}

