/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: compat_memory.h                                             *
 *                                                                     *
 * Description:                                                        *
 *  Compatibiliy defines and macros for work with memory from          *
 *  module.                                                            *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __COMPAT_MEMORY_H__
#define __COMPAT_MEMORY_H__

#include <linux/mm.h>

#ifndef page_to_pfn
# define page_to_pfn(page) ((unsigned long)((page) - mem_map))
#endif

#ifndef pfn_to_page
# define pfn_to_page(pfn) (mem_map + (pfn))
#endif

#ifndef pte_offset_kernel
# define pte_offset_kernel pte_offset
#endif

#ifndef pte_offset_map
# define pte_offset_map pte_offset
#endif

#ifndef pte_pfn
# define pte_pfn(pte) (pte_val(pte) >> PAGE_SHIFT)
#endif

/* get aphysical page by address */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 19)
static inline unsigned int mmGetPhyByAddr(unsigned int addr) {

    pgd_t *pgd;
    pmd_t *pmd;
    pte_t *ptep, pte;

    pgd = pgd_offset(current->mm, addr);
    if (pgd_none(*pgd) || pgd_bad(*pgd))
        goto out;

    pmd = pmd_offset(pgd, addr);
    if (pmd_none(*pmd) || pmd_bad(*pmd))
        goto out;

    ptep = pte_offset(pmd, addr);
    if (!ptep)
        goto out;

    pte = *ptep;
    if (pte_present(pte)) {
            return pte_pfn(pte);
    }

out:
    return 0;
}
#endif

/* pgd compat */
#ifdef PUD_MASK
# define compat_pgd_present(x)      pud_present(x)
typedef pud_t  compat_pgd_t;
#elif defined(pml4_offset)
# define compat_pgd_present(x)      pml4_present(x)
typedef pml4_t compat_pgd_t;
#else
# define compat_pgd_present(x)      pgd_present(x)
typedef pgd_t  compat_pgd_t;
#endif

/* invalidate page */
#define invalidate_page(page)   \
    __asm__ __volatile__ (      \
        "movl %0, %%eax\n\t"    \
        "invlpg (%%eax)\n\t"    \
        : : "r" (page)          \
        : "%eax"                )

/* Page lock */
#ifndef SetPageLocked
# ifndef LockPage
#  define compat_SetPageLocked(x) set_bit(PG_locked, &(x)->flags)
# else
#  define compat_SetPageLocked(x) LockPage(x)
# endif
#else
# define compat_SetPageLocked(x) SetPageLocked(x)
#endif

/* Page unlock */
#ifndef ClearPageLocked
# ifndef ClearLockPage
#  define compat_ClearPageLocked(x) clear_bit(PG_locked, &(x)->flags)
# else
#  define compat_ClearPageLocked(x) ClearLockPage(x)
# endif
#else
# define compat_ClearPageLocked(x) ClearPageLocked(x)
#endif

/* change page attr */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 20)
# define compat_change_page_attr(pg, cnt, flag) change_page_attr(pg, cnt, flag)
#else
# define compat_change_page_attr(pg, cnt, flag) (0)
#endif

/* global flush tlb */
#define COMPAT_X86_CR4_PGE             0x0080
#define compat_global_flush_tlb()                           \
    do {                                                    \
        unsigned int tmpreg, cr4, cr4_orig;                 \
                                                            \
        __asm__ __volatile__(                               \
            "movl %%cr4, %2;  \n"                           \
            "movl %2, %1;     \n"                           \
            "andl %3, %1;     \n"                           \
            "movl %1, %%cr4;  \n"                           \
            "movl %%cr3, %0;  \n"                           \
            "movl %0, %%cr3;  \n"                           \
            "movl %2, %%cr4;  \n"                           \
            : "=&r" (tmpreg), "=&r" (cr4), "=&r" (cr4_orig) \
            : "i" (~COMPAT_X86_CR4_PGE)                     \
            : "memory");                                    \
        } while (0)
    
#endif /* __COMPAT_MEMORY_H__*/
