/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: compat_smp.h                                                *
 *                                                                     *
 * Description:                                                        *
 *  Various SMP stuff.                                                 *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __COMPAT_SMP_H__
#define __COMPAT_SMP_H__

#include <linux/autoconf.h>
#include <linux/version.h>
#include <linux/smp.h>

/* check for preemption */
#ifdef CONFIG_PREEMPT
# include <linux/preempt.h>
#else
# ifndef preempt_disable
#  define preempt_disable() do { } while(0)
# endif
# ifndef preempt_enable
#  define preempt_enable()  do { } while(0)
# endif
#endif

/* call on each cpu function */
static inline int compat_smp_ipi_call(void (* fn)(void *arg), void *arg, int retry, int wait) {

#ifdef CONFIG_SMP
    int ret;
#endif

    preempt_disable();

#ifdef CONFIG_SMP
    if (((ret = smp_call_function(fn, arg, retry, wait)) != 0)) { 
        Err("Smp call error: %d", ret);
        return -EFAULT; 
    }
#endif /* CONFIG_SMP */
    fn(arg);

    preempt_enable();
    
    return 0;
}

#endif /* __COMPAT_SMP_H__ */

