/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: ioctls.h                                                    *
 *                                                                     *
 * Description:                                                        *
 *  Defines for ioctl commands                                         *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __IOCTLS_H__
#define __IOCTLS_H__

#ifndef STD_INTERCOM
# define STD_INTERCOM
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* ioctl's macros */
#ifndef _BR_IOC_NRBITS
#define _BR_IOC_NRBITS      8
#define _BR_IOC_TYPEBITS    8
#define _BR_IOC_SIZEBITS    14
#define _BR_IOC_DIRBITS     2

#define _BR_IOC_NRSHIFT     0
#define _BR_IOC_TYPESHIFT   (_BR_IOC_NRSHIFT+_BR_IOC_NRBITS)
#define _BR_IOC_SIZESHIFT   (_BR_IOC_TYPESHIFT+_BR_IOC_TYPEBITS)
#define _BR_IOC_DIRSHIFT    (_BR_IOC_SIZESHIFT+_BR_IOC_SIZEBITS)

#define _BR_IOC(dir,type,nr,size)    \
    (((dir)  << _IOC_BR_DIRSHIFT) |  \
     ((type) << _IOC_BR_TYPESHIFT) | \
     ((nr)   << _IOC_BR_NRSHIFT) |   \
     ((size) << _IOC_BR_SIZESHIFT))

#define _BR_IOC_WRITE   1U
#define _BR_IOC_READ    2U
#define _BR_IOWR(type, nr, size) \
        _IOC(_BR_IOC_READ | _BR_IOC_WRITE, type, nr, sizeof(size))
#endif

#define CTL_CODE(cmd)  ((0x8F010000) | ((cmd) << 2))

/* request structure */
struct ioctl_request_t {
    /* Code of command */
    unsigned CtlCode;
    /* Number of bytes in input buffer */
    unsigned InLength;
    /* Number of bytes in output buffer */
    unsigned OutLength;
    /* Pointer to the exchange data buffer  */
    void *Buffer;
    /* Number of bytes returned */
    unsigned ReturnCount;
    /* Pointer to the list of allocated memory buffers which are neccesary for VM execution */
    void *GMemInfoBuf;
};

/* struct for initialize PMM */
struct pmm_initial_values_t {
    unsigned int GuestMemBase;
    unsigned int PmmShareBuff;
    unsigned int PmmShareBuffSize;
};

/* monitor function description */
struct monitor_functions_def_t {
    /* Identifier of the function */
    unsigned int fId;
    /* Name of the function */
    char fName[15];
};

/* Identifiers of the Monitor functions */
enum {
    /* Open connection with Monitor */
    MONFUNC_OPEN,
    /* Close connection with Monitor */
    MONFUNC_CLOSE,
    /* Process request to Monitor */
    MONFUNC_CONTROL,
    /* Process fast request to Monitor */
    MONFUNC_REQUEST,
    /* Number of import functions */
    MONFUNC_COUNT
};

/* Enumeration for monitor and driver requests */
#define CMD_BASE_INDEX      0x801
enum {
    CMD_QUIT = CMD_BASE_INDEX,
    CMD_INIT_MONITOR,
#ifndef CALL_MONITOR_FROM_GATE
    CMD_EXCHG_REQUEST, 
#endif
    
    CMD_GUEST_CPU_INFO,
    CMD_UPDATE_GUEST_MEM,
    CMD_PREPARE_MEM_INFO,
    CMD_MAP_MEM,
    CMD_SETUP_MON_FUNCTIONS,
    CMD_MODE_FEATURES,
#ifdef CALL_MONITOR_FROM_GATE
    CMD_SET_INT_GATE,
    CMD_CLR_INT_GATE,
#endif

    CMD_PMM_LOCK_PAGE,
    CMD_PMM_UNLOCK_PAGE,
    CMD_PMM_INIT,
    CMD_PMM_UNINIT,
    CMD_PMM_SET_MAX_MEMORY_FOR_VM,
    CMD_ALLOC_SHAREBUFF_FOR_PMM,
    CMD_FREE_SHAREBUFF_FOR_PMM,
    CMD_REPRESENT_MEM,
    CMD_RELEASE_MEM
};

/* four global ioctls type */
#define VM_MAIN_REQUEST         _BR_IOWR('T', 1, sizeof(struct ioctl_request_t))
#define VM_SET_TIMER_RATE       _BR_IOWR('T', 2, sizeof(unsigned))
#define VM_SET_TIMER_ALARM      _BR_IOWR('T', 3, sizeof(unsigned))
#define VM_CHECK_RANDOM_SALT    _BR_IOWR('T', 4, sizeof(struct ioctl_request_t))
#define VM_REGISTER_IDT         _BR_IOWR('T', 5, sizeof(void))
#define VM_DEREGISTER_IDT       _BR_IOWR('T', 6, sizeof(void))
#define VM_TRY_LOCK_DRIVER      _BR_IOWR('T', 7, sizeof(void))
#define VMX_CHECK               _BR_IOWR('T', 8, sizeof(unsigned int))
#define HYP_IDTR_CHECK          _BR_IOWR('T', 9, sizeof(unsigned int))

/* subrequests for VM_MAIN_REQUEST */
/* monitor init/quit */
#define IOCTL_INIT_MONITOR                  CTL_CODE(CMD_INIT_MONITOR)
#define IOCTL_QUIT                          CTL_CODE(CMD_QUIT)

#ifndef CALL_MONITOR_FROM_GATE
# define IOCTL_EXCHG_REQUEST                 CTL_CODE(CMD_EXCHG_REQUEST)
#endif

/* some generic ioctls */
#define IOCTL_GUEST_CPU_INFO                CTL_CODE(CMD_GUEST_CPU_INFO)
#define IOCTL_UPDATE_GUEST_MEM              CTL_CODE(CMD_UPDATE_GUEST_MEM)
#define IOCTL_MAP_MEM                       CTL_CODE(CMD_MAP_MEM)
#define IOCTL_PREPARE_MEM_INFO              CTL_CODE(CMD_PREPARE_MEM_INFO)
#define IOCTL_MODE_FEATURES                 CTL_CODE(CMD_MODE_FEATURES)

/* Int Gate ioctls */
#define IOCTL_SET_INT_GATE                  CTL_CODE(CMD_SET_INT_GATE)
#define IOCTL_CLR_INT_GATE                  CTL_CODE(CMD_CLR_INT_GATE)

/* memory manaer ioctls */
#define IOCTL_PMM_INIT                      CTL_CODE(CMD_PMM_INIT)
#define IOCTL_PMM_UNINIT                    CTL_CODE(CMD_PMM_UNINIT)
#define IOCTL_PMM_LOCK_PAGE                 CTL_CODE(CMD_PMM_LOCK_PAGE)
#define IOCTL_PMM_UNLOCK_PAGE               CTL_CODE(CMD_PMM_UNLOCK_PAGE)
#define IOCTL_PMM_SET_MAX_MEMORY_FOR_VM     CTL_CODE(CMD_PMM_SET_MAX_MEMORY_FOR_VM)
#define IOCTL_ALLOC_SHAREBUFF_FOR_PMM       CTL_CODE(CMD_ALLOC_SHAREBUFF_FOR_PMM)
#define IOCTL_FREE_SHAREBUFF_FOR_PMM        CTL_CODE(CMD_FREE_SHAREBUFF_FOR_PMM)

/* monitor loading specific */
#define IOCTL_REPRESENT_MEM                 CTL_CODE(CMD_REPRESENT_MEM)
#define IOCTL_RELEASE_MEM                   CTL_CODE(CMD_RELEASE_MEM)
#define IOCTL_SETUP_MON_FUNCTIONS           CTL_CODE(CMD_SETUP_MON_FUNCTIONS)

/* borders of ioctl commands */
#define END_IOCTL                           CTL_CODE(CMD_RELEASE_MEM)
#define START_IOCTL                         CTL_CODE(CMD_QUIT)

#define LOCK_PAGE_STATUS_SUCCESS	0x0
#define LOCK_PAGE_STATUS_FAIL		0xffffffff
#define LOCK_PAGE_STATUS_ONLY_HUGE	0xfffffffe
#define IS_LOCK_FAILED(uResult)		(uResult & 0x80000000)

struct LOCK_PAGE_RESULT
{
	UINT uRequiredSize; //-1 if the size is getting from shared page
	UINT uResult;
};

#ifdef __cplusplus
}
#endif

#endif /* __IOCTLS_H__ */
