/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: collector.c                                                 *
 *                                                                     *
 * Description:                                                        *
 *  Physical pages collector, it will handle all pages allocations     *
 *  and deallocations. Write it to cache and if application is died    *
 *  this code cleanup all pages and free all memory allocated in       *
 *  kernel.                                                            *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/mm.h>
#include <linux/slab.h>

#include "vmmain.h"
#include "mm.h"
#include "compat_procs.h"
#include "compat_memory.h"
#include "logging.h"

/* cached page slab */
struct vm_cached_page {
    struct task_struct *tp;  /* pointernter to task_struct */
    struct page *pg;         /* pointer to page owner */
};

/* page lock */
inline int vm_mmGetPage(struct page *pg, struct bma_t *bma) {

    if (!pg || !bma)
        return 0;

    if (!bma_test(bma, page_to_pfn(pg))) {
        bma_set(bma, page_to_pfn(pg), 1);
        vmGlobal.vmWset.totalAllocated ++;
        return 0;
    }

    return 1;
}

/* page unlock */
inline void vm_mmPutPage(struct page *pg, struct bma_t *bma) {

    if (!pg)
        return;

    if (!bma) {
        /* in some cases this will happen. 
         * (on driver unload with dead apps), but this is a problem ! 
         * --ms
         */
        Warn("Put with NULL bma !");
        dump_stack();
        put_page(pg);
    } else {
        if (bma_test(bma, page_to_pfn(pg))) {
            bma_set(bma, page_to_pfn(pg), 0);
            put_page(pg);
        }
    }
    vmGlobal.vmWset.totalAllocated --;
}

/* free pages from current task */
void QUICKCALL vm_mmFreeCurrent(struct bma_t *bma) {

    struct page *pg;

    if (!bma)
        return;

    BMA_START_FOREACH(bma);

    while (bma_foreach(bma) != -1) {
        bma_set(bma, BMA_GET_FPOS(bma), 0);
        pg = pfn_to_page(BMA_GET_FPOS(bma));
        if (pg) {
            put_page(pg);
            vmGlobal.vmWset.totalAllocated --;
        }
    }

    bma_deinit(bma);
}
