/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: wset.c                                                      *
 *                                                                     *
 * Description:                                                        *
 *  Workingset memory usage balancer.                                  *
 *                                                                     *
 * Comments:                                                           *
 *  Based on old code from misha@.                                     *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include "Spinlock.h"
#include "vmmain.h"
#include "mm.h"
#include "logging.h"

#define VMINTSIZE 32
#define MIN_PAGES_FOR_VM 256

/* reorder wset structures */
static void reorderWset(void) {

    long tPages;

    down(&vmGlobal.vmWset.wsSem);

    /* avreage count of memory regions */
    tPages = (vmGlobal.vmWset.overallMemoryLimit / 4) * sizeof(struct pmm_region_t);

    tPages = tPages / PAGE_SIZE;

    /* save value */
    vmGlobal.vmWset.regPages = tPages;

    /* count pages used by all vms and average value for regions */
    tPages = (VMINTSIZE * 256) * vmGlobal.vmWset.wsCount;
    tPages += vmGlobal.vmWset.regPages;

    /* save value of available pages for vm's */
    vmGlobal.vmWset.vmsPages = vmGlobal.vmWset.overallMemoryLimit - tPages;

    up(&vmGlobal.vmWset.wsSem);
}

int QUICKCALL calcWset(void) {

    struct list_head *q;
    long eachWS;
    struct pmm_struct_t *pmm;
    int i;
    
    if(!vmGlobal.vmWset.wsCount)
        return 0;

    reorderWset();
    
    if(vmGlobal.vmWset.vmsPages < MIN_PAGES_FOR_VM)
        return -1;

    eachWS = vmGlobal.vmWset.vmsPages / vmGlobal.vmWset.wsCount;
    if (eachWS < MIN_PAGES_FOR_VM)
        return -1;

    /* lock workingset access */    
    down(&vmGlobal.vmWset.wsSem);
    vmGlobal.vmWset.allVmsUsed = 0;

    /* lock pmm's list */
    down(&vmGlobal.vmPmms.pmmAccSem);

    /* walk trought list of pmms */
    i = 0;
    list_for_each(q, &vmGlobal.vmPmms.pmmList) {
        i++;
        pmm = list_entry(q, struct pmm_struct_t, list);
        
        Lock(&pmm->MonitorPmmBuff->WSSpinLock);

        /* is current VM ram lower then eachWS limit */
        if(eachWS > pmm->MonitorPmmBuff->uGuestRamPages) {
            vmGlobal.vmWset.allVmsUsed += pmm->MonitorPmmBuff->uGuestRamPages;

            /* reorder eachWS limit */
            if(vmGlobal.vmWset.wsCount != i) {
                eachWS = (vmGlobal.vmWset.vmsPages - vmGlobal.vmWset.allVmsUsed) / (vmGlobal.vmWset.wsCount - i);
            }

            /* adjust */
            pmm->MonitorPmmBuff->iAdjustWorkingSet = (pmm->MonitorPmmBuff->uGuestRamPages - pmm->MonitorPmmBuff->uMaxPages);
        } else {
            vmGlobal.vmWset.allVmsUsed += eachWS;
        
            /* adjust */
            pmm->MonitorPmmBuff->iAdjustWorkingSet = (eachWS - pmm->MonitorPmmBuff->uMaxPages);
        }
        Unlock(&pmm->MonitorPmmBuff->WSSpinLock);
    }
    vmGlobal.vmWset.eachWS = eachWS;

    /* unlock */
    up(&vmGlobal.vmPmms.pmmAccSem);
    up(&vmGlobal.vmWset.wsSem);

    return 0;
}
