/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: monexport.c                                                 *
 *                                                                     *
 * Description:                                                        *
 *  Fill structure with exported funstions to monitor.                 *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/vmalloc.h>

#include "vmmain.h"
#include "logging.h"

#include "compat_procs.h"

int vmSetExports(struct module_param_t *param) {

    if (!param)
        return -EINVAL;
    
    if (!param->MonAddr)
        return -EINVAL;
        
    /* Start address and size of Monitor region */
    param->drvInfo.monAddr = param->MonAddr;
    param->drvInfo.monSize = param->MonSize;
    
    /* Export functions */
    param->drvInfo.exportedFunc.logFunction        = logMonitorCallback;
    param->drvInfo.exportedFunc.allocMem           = mm_mon_allocMem;
    param->drvInfo.exportedFunc.freeMem            = mm_mon_freeMem;
    param->drvInfo.exportedFunc.allocContigMem     = mm_mon_allocContigMem;
    param->drvInfo.exportedFunc.freeContigMem      = mm_mon_freeContigMem;
    param->drvInfo.exportedFunc.representToMonMem  = mm_mon_representToMonMem;
    param->drvInfo.exportedFunc.releaseFromMonMem  = mm_mon_releaseFromMonMem;
    
    param->drvInfo.exportedFunc.getPagesIndex      = mm_mon_getPagesIndex;

#ifndef CALL_MONITOR_FROM_GATE
    param->drvInfo.exportedFunc.redirectIRQ        = vm_mon_redirectIRQ;
#endif
    
    param->drvInfo.memBuff = NULL;
    
    return 0;
}

#ifndef CALL_MONITOR_FROM_GATE
/* irq redirection */
long STD_INTERCOM vm_mon_redirectIRQ(unsigned int irq) {

#ifdef SLEEP_AVG
    current->sleep_avg = 0;
#endif
   
    return compat_need_resched();
    
}
#endif
