/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: vmmain.c                                                    *
 *                                                                     *
 * Description:                                                        *
 *  Main module file. Initialization/deinitialization routines are     *
 *  living here.                                                       *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#define MAIN_MODULE_FILE
#define MODULE_DESCRIPTION_STRING "Main module for Parallels virtual machine"

#include "vmmodule.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
# include <linux/moduleparam.h>
#endif

#include "vmmain.h"
#include "logging.h"
#include "timer.h"
#include "intgate.h"

#include "utils.h"

extern int hypervisorPresentInSystem(void);

/* salt */
char *rsalt;

#ifdef module_param
 module_param(rsalt, charp, 0);
#else
 MODULE_PARM(rsalt, "0-1023s");
#endif

/* module init/deinit */
int init_module(void) {

    hypervisorPresentInSystem(); /* fake call to hypervisor module -- ms */
    
    /* initialize salt */
#ifdef ENABLE_MODULES_PROTECT
    if (!rsalt) {
        Err("No salt specified!");
        return -1;
    }
    vmGlobal.salt = rsalt;
#endif

    /* initialize /dev/vm-main device */
    if (vmDeviceInit())
        return -1;

    /* initialize proc fs */
    if (vmProcInit())
        return -1;

    /* init pmm */
    if (vmPmmsMainInit())
        return -1;

    /* setup wset */
    vmWsetSetup();

    /* init IntGate */
    ig_InitIntGate();

    Log("%s %s-%d Main module initialized.", TARGET_NAME_DISP, TARGET_VERSION, VER_BUILD);

    /* init some values */
    vmGlobal.vmPmms.totalMem = 0;
    
    return 0;
}

void cleanup_module(void) {

    /* cleanup proc */
    vmProcDeinit();

    /* remove device */
    vmDeviceDeinit();

    /* deinit pmms */
    vmPmmsMainDeinit();

    /* stop timer */
    vmStopTimer();

    Log("%s %s-%d Main module exited.", TARGET_NAME_DISP, TARGET_VERSION, VER_BUILD);
}
