/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: vmmain.h                                                    *
 *                                                                     *
 * Description:                                                        *
 *  Main defines file for module                                       *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __VMMAIN_H__
#define __VMMAIN_H__

#include <linux/slab.h>
#include "timer.h"

#include "vm_compiler.h"
#include "ioctls.h"

#include "Types.h"
#include "PMM.h"

/* proc fs */
int vmProcInit(void);
void vmProcDeinit(void);

/* device */
int vmDeviceInit(void);
void vmDeviceDeinit(void);

/* pages cache */
int vmPcacheInit(void);
void vmPcacheDeinit(void);

/* internal data */
struct module_param_t *iDataOpen(void);
void iDataClose(struct module_param_t *);
void iDataFree(struct module_param_t *);


/* monitor exported functions */
void STD_INTERCOM      *mm_mon_allocMem(long);
void STD_INTERCOM       mm_mon_freeMem(void *);
void STD_INTERCOM      *mm_mon_allocContigMem(long);
void STD_INTERCOM       mm_mon_freeContigMem(void *, long);
long STD_INTERCOM       mm_mon_representToMonMem(void *, long, void **, void **, void *);
void STD_INTERCOM       mm_mon_releaseFromMonMem(void *, void *, void *);
unsigned STD_INTERCOM   mm_mon_getPagesIndex(unsigned long *, int, void *, unsigned, void *);
#ifndef CALL_MONITOR_FROM_GATE
long STD_INTERCOM       vm_mon_redirectIRQ(unsigned int);
#endif

/* exports manager */
int vmSetExports(struct module_param_t *);

/* Status definition */
typedef long NTSTATUS;
#define STATUS_SUCCESS      ((NTSTATUS)0x00000000L)
#define STATUS_UNSUCCESSFUL ((NTSTATUS)0xC0000001L)
#define NT_SUCCESS(Status)  ((NTSTATUS)((Status & 0x80000000) == 0))

/* monitor function typedef */
typedef NTSTATUS (STDCALL * monitor_funct_t)(void *, int, void *);

/* functions exported to monitor */
struct extern_funcs_t {
    /* Output debug information  */
    void  STD_INTERCOM      (*logFunction)(char *, ...);
    /* Allocation memory */
    void* STD_INTERCOM      (*allocMem)(long);
    /* Release memory */
    void  STD_INTERCOM      (*freeMem)(void *);
    /* Allocation contig memory */
    void* STD_INTERCOM      (*allocContigMem)(long);
    /* Release contig memory */
    void  STD_INTERCOM      (*freeContigMem)(void *, long);
    /* Representation neccesary Application address area to Monitor
     * Memory lock and mapping */
    long  STD_INTERCOM      (*representToMonMem)(void *, long, void **, void **, void *);
    /* Function cancels fixed memory representation to Monitor address area
     * Memory unlock and unmapping */
    void  STD_INTERCOM      (*releaseFromMonMem)(void *, void *, void *);
    /* Gather indexes of physical pages for virtual address area */
    unsigned STD_INTERCOM   (*getPagesIndex)(unsigned long *, int, void *, unsigned, void *);
#ifndef CALL_MONITOR_FROM_GATE
    /* redirect IRQ */
    long STD_INTERCOM (*redirectIRQ)(unsigned int);
#endif
};


/***************** Exports *****************/
/* exported directly from monitor */
typedef struct MON_MEM_DESC {
    unsigned int            uPages;
    struct page             **pPages;
    struct vm_area_struct   **vmas;
    unsigned int            pMonAddr;
    unsigned int            bValid;
} MON_MEM_DESC;

/* exported from Api.h */
typedef struct MEM_MAP {
    void        *m_pMemPtr;
    void        *m_pMdlPtr;
    void        *m_pMonPtr;
} MEM_MAP;

typedef struct MEMORY_INFO {
    int             m_MemType;
    unsigned int    m_dwMemSize;
    int             m_bUserMap;
    MEM_MAP         m_MemMap;
} MEMORY_INFO;

/* monbody */
#define SYS_MONBODY_MEMORY 201

/* driver info structure */
struct driver_info_to_mon_t {
    /* monitor start address */
    void *monAddr;
    /* monitor size */
    unsigned monSize;
    /* functions exported to monitor */
    struct extern_funcs_t exportedFunc;
    /* exchange buffer between monitor and driver */
    void *memBuff;
};

/***************** Exports *****************/

/* module info struct */
struct module_param_t {

    /* Pointer to the start address of Monitor region */
    void *MonAddr;

    /* Size of Monitor region */
    unsigned MonSize;

    /* driver info for monitor */
    struct driver_info_to_mon_t drvInfo;

    /* hires timer */
    struct vmTimer timer;

    /* is salt passed ? */
    int saltPassed;
    
    /* exported old stub class */
    struct {
        /* List of Monitor functions */
        monitor_funct_t MonitorFuncs[MONFUNC_COUNT];

        /* Pointer to the Monitor state */
        void *pvm;

        /* main PMM struct  */ 
        struct pmm_struct_t *PmmData;

        /* pmm share buffer saved pointer */
        void *pmmShareBuff;

        /* bma phy pages structure */
        struct bma_t *bma;

        /* IntGate monitor id */
        unsigned int MonId;
    } iData;
};

/* global structure */
struct vm_global_t {

    /* array of pages cache */
    struct farray_head *phyPagesCache;

    /* random salt */
    char *salt;
    
    struct {
        /* pmm's access mutex */
        struct semaphore pmmAccSem;
        /* list of PMM's */
        struct list_head pmmList;
        /* regions cache */
        kmem_cache_t *pmmRegionsSlab;
        /* total guest memory */
        unsigned long totalMem;
    } vmPmms;

    struct {
        /* max memory limit */
        long overallMemoryLimit;
        /* semaphore for working set calculations */
        struct semaphore wsSem;
        /* Workstations count */
        int wsCount;
        /* pages used by regoins descriptors */
        long regPages;
        /* pages available for vms */
        long vmsPages;
        /* used by all vms */
        long allVmsUsed;
        /* each WS */
        long eachWS;
        /* total pages allocated by all apps */
        long totalAllocated;
    } vmWset;
};

#ifdef MAIN_MODULE_FILE
 UNUSED struct vm_global_t vmGlobal;
#else
 extern struct vm_global_t vmGlobal;
#endif

#include "mm.h"

#endif /* __VMMAIN_H__ */
