/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: ioctls.c                                                    *
 *                                                                     *
 * Description:                                                        *
 *  Hypervisor ioctl() calls handler.                                  *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#include "vmmodule.h"

#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/spinlock.h>
#include <asm/uaccess.h>

#include "ioctls.h"
#include "logging.h"
#include "vtx_init.h"
#include "compat_smp.h"

#include "hypervisor.h"
#include "x86.h"

/* applications counter and control stuff */
extern int driverLocked;
extern int appCounter;
extern int vtSupport;
extern spinlock_t appLock;

void hypIdtrCheck(void *p)
{
	struct idt_info_t Idtr;
	int nCpu;

	do_sidt(Idtr);
	nCpu = smp_processor_id();

	if (Idtr.pBase != hypState.hypIdtrList[nCpu].pBase)
	{
		do_lidt(hypState.hypIdtrList[nCpu]);
	}
}

/* ioctl()'s calls handler for hypervisor device */
int hypIoctl(struct inode *ino, struct file *fp, unsigned int iocmd, unsigned long ioarg) {

    int ret = -EINVAL;
    int iVtxSupport;
    void *arg = (void *)ioarg;
    struct idt_info_t Idtr;
    int nCpu;
   
    switch(iocmd) {
        case VMX_CHECK:
            if (!copy_to_user(arg, &vtSupport, sizeof(vtSupport)))
                ret = 0;
    	    break;
        case HYP_IDTR_CHECK:
            preempt_disable();
	    do_sidt(Idtr);
	    nCpu = smp_processor_id();

	    if (Idtr.pBase != hypState.hypIdtrList[nCpu].pBase)
	    {
		if ( (ret = smp_call_function(hypIdtrCheck, NULL, 1, 1)) )
		{
			printk("[hypIdtrCheck] SMP call error: %i\n", ret);
			return -EFAULT;
		}
		do_lidt(hypState.hypIdtrList[nCpu]);
	    }
	    preempt_enable();
    	    break;
        default:
            Err("Unsupported ioctl() call (0x%x) to hypervisor", iocmd);
            break;
    }
    
    return ret;
}
