/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: hypercall.h                                                 *
 *                                                                     *
 * Description:                                                        *
 *  Hypercall interface.                                               *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __HYPERCALL_H__
#define __HYPERCALL_H__

#include "vm_compiler.h"

#include "intgate.h"

#ifdef _HYPERVISOR_
# include "hypvmstate.h"

# include "x86.h"
#endif /* _HYPERVISOR_ */

#define HYPERCALL_INT 27 /* hypercall interrupt number */

int QUICKCALL hypHypercallInit(int);
void QUICKCALL hypHypercallDeinit(int);

/* hypercalls */
enum {
    HYP_ASSIGN_INTERRUPT,
    HYP_CREATE_VM,
    HYP_DELETE_VM,
    HYP_ACTIVATE_VM,
    HYP_GETVM_CODE_ADDR,
    HYP_GETVM_IDT_ADDR,
    HYP_GETVM_CODE_PHY_PAGES,
    HYP_GETVM_IDT_PHY_PAGES,
    HYP_GET_INTHANDLERS_INFO,
    HYP_REGISTER_INT_HANDLER,
    HYP_GET_PROS_PAGING_MODE,

    HYP_CALLS_COUNT,
};

/* three temporary magic numbers */
#define MAGIC_NUM1 0x6d616769
#define MAGIC_NUM2 0x63216e75
#define MAGIC_NUM3 0x6d626572

/* internal hypervisor declares */
#if defined(_HYPERVISOR_) || defined(_HYPER_BINARY_PART_)

 /* hypercalls */
 int QUICKCALL hypAssignInterrupt_Call(unsigned int, unsigned int, struct idt_descriptor_t *);

 void* QUICKCALL hypCreateVM_Call(void);
 
 int QUICKCALL hypDeleteVM_Call(struct hyp_vmstate_t *);
 
 int QUICKCALL hypActivateVM_Call(struct hyp_vmstate_t *);
 
 unsigned int QUICKCALL hypGetVMCodeAddr_Call(struct hyp_vmstate_t *);
 
 unsigned int QUICKCALL hypGetVMIdtAddr_Call(struct hyp_vmstate_t *);
 
 int QUICKCALL hypGetVMCodePhyPages_Call(struct hyp_vmstate_t *, unsigned int *, unsigned int, unsigned int *);
 
 int QUICKCALL hypGetVMIdtPhyPages_Call(struct hyp_vmstate_t *, unsigned int *, unsigned int, unsigned int *);
 
 int QUICKCALL hypGetIntHandlerInfo_Call(struct hyp_vmstate_t *, struct int_handlers_info_t *);
 
 int QUICKCALL hypRegisterIntHandler_Call(struct hyp_vmstate_t *, unsigned int, struct idt_descriptor_t *);

 unsigned int QUICKCALL hypGetPrOsPagingMode_Call(void);

#endif /* _HYPERVISOR_ */

/* assign interrupt call */
static inline int hypAssignInterrupt(unsigned int intNo, unsigned int cpuNo, void *gate) {

    int ret;
    
    __asm__ __volatile__ (
        "pushal\n\t"    
        "movl %6, %%esi\n\t"
        "movl %7, %%edi\n\t"
        "movl %8, %%ebx\n\t"
        "pushl %1\n\t"
        "pushl %2\n\t"
        "pushl %3\n\t"
        "movl %%esp, %%edx\n\t"
        "movl %5, %%eax\n\t"
        "int %4\n\t"
        "addl $12, %%esp\n\t"
        "movl %%eax, %0\n\t"
        "popal\n\t"
        : "=r" (ret)
        : "r" (gate), "r" (cpuNo), "r" (intNo),
          "nN" (HYPERCALL_INT), "nN" (HYP_ASSIGN_INTERRUPT),
          "nN" (MAGIC_NUM1), "nN" (MAGIC_NUM2), "nN" (MAGIC_NUM3)
        : "%ebx", "%edi", "%esi"
    );
    
    return ret;
}

#endif /* __HYPERCALL_H__ */

