/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: hypervisor.h                                                *
 *                                                                     *
 * Description:                                                        *
 *  Hypervisor common include file.                                    *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __HYPERVISOR_H__
#define __HYPERVISOR_H__

#ifdef _HYPERVISOR_

#include "vm_compiler.h"

#ifndef _HYPER_BINARY_PART_
# include <linux/mm.h>
# include <linux/slab.h>

# include <asm/semaphore.h>

# include "mm.h"
# include "intgate.h"
#endif

#include "hypvmstate.h"

#include "interrupt.h"

#endif /* _HYPERVISOR_ */

/* init/deinit internal hypervisor structs */
int QUICKCALL hypInternalInit(void);
void QUICKCALL hypInternalDeinit(void);

/* exceptions handlers */
void QUICKCALL hypHandleExceptionWithError(void);
void QUICKCALL hypHandleExceptionWithoutError(void);
void QUICKCALL hypHandleExceptionInterrupt(void);
void QUICKCALL hypHandleExceptionHyperCall(void);

/* primary os idt routines */
int QUICKCALL hypAllocateIdt(void);
int QUICKCALL hypFreeIdt(void);
int QUICKCALL hypRegisterIdt(int);
int QUICKCALL hypDeregisterIdt(int);

/* hypervisor device init/deinit routines */
int QUICKCALL hypDeviceInit(void);
void QUICKCALL hypDeviceDeinit(void);

/* ipi handler */
void QUICKCALL hypIpiHandler(void *);

/* get exception type */
int QUICKCALL hypGetExcType(int);

/* count processors */
int QUICKCALL hypCountProcessors(void);
    
#define INT_HANDLER_SIZE   64 

/* IDT stub */
typedef union {
    unsigned char handlerCode[INT_HANDLER_SIZE];
} PACKED int_handler_t;

#ifdef _HYPERVISOR_
//# include "x86.h"
 int QUICKCALL hypSetIntHandler(int, int, int, int_handler_t *, unsigned int);
#endif

/* interrupt types */
enum {
    IDT_INTERRUPT,
    IDT_EXCEPTION_ERROR,
    IDT_EXCEPTION_NOERROR,
};

/* request types for ipi handler */
enum {
    IPI_REGISTER_IDT,
    IPI_DEREGISTER_IDT,
    IPI_COUNT_CPUS,
};

#ifdef _HYPERVISOR_

# include "hypidtemul.h"

/* hypervisor internal structure */
struct hypervisor_state_t {

    /* hypervisor idt tables */
    struct idt_info_t hypIdtrList[MAX_PROCESSOR_NUMBER];
    
    /* primary os idt tables */
    struct idt_info_t hosIdtrList[MAX_PROCESSOR_NUMBER];
    
    /* list of pointers to hypervisor idt pages */
    void *hypIdtrTablesList[MAX_PROCESSOR_NUMBER];
    
    /* list of hypervisor handlers code */
    int_handler_t *hypIntHandlersList[MAX_PROCESSOR_NUMBER];

    /* emulation data */
    struct idt_emulation_t idtEmul;
    
    /* number of detected cpus */
    int cpuCount;

    /* error on init */
    int initError;

    /* primary OS paging mode (see constants from PROS_PAGING_MODE) */    
    unsigned int uPrOsPagingMode;

#ifndef _HYPER_BINARY_PART_
    /* slab cache for vmstates */
    kmem_cache_t *vmStateCache;

    /* list of vmstates */
    struct list_head vmStateList;

    /* vmstates list access semaphore */
    struct semaphore vmsListAccSem;

    /* file pointer to /dev/hypervisor */
    struct file *hypFp;
#endif

} ALIGNED;

#ifdef MAIN_MODULE_FILE

 /* global hypState */
 struct UNUSED hypervisor_state_t hypState;

 /* flag for check reentry to interrupts handlers */
 unsigned int *hypInHandlerFlag;

#else 
 /* global hypState */
 extern struct hypervisor_state_t hypState;

 /* flag for check reentry to interrupts handlers */
 extern unsigned int *hypInHandlerFlag;
#endif

/* some independ stuff for binary part */
#ifdef _HYPER_BINARY_PART_
# ifndef __KERNEL__
#  define __KERNEL__
# endif
# include <linux/string.h>  /* for memset/cpy and family */
# include <asm/page.h>      /* for PAGE_SIZE */
# include "mm.h"

 /* some wrappers */
 int QUICKCALL hyper_wrap_smp_processor_id(void);
 unsigned int QUICKCALL hyper_wrap_page_to_pfn(struct page *);

#endif /* _HYPER_BINARY_PART_ */

#endif /* _HYPERVISOR_ */

#endif /* __HYPERVISOR_H__ */
