/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: hypidtemul.h                                                *
 *                                                                     *
 * Description:                                                        *
 *  Hypervisor IDT access emulation.                                   *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __HYPIDTEMUL_H__
#define __HYPIDTEMUL_H__

#include "intgate.h"

/* maximum number of redirects */
#define MAX_REDIRECT_CHAIN_LENGTH 16

/* main struct for each idt entry */
struct idt_entry_state_t {
    /* entry is used */
    int dirty;
    
    /* flags and vlues  */
    unsigned char entryBytes[8];
    unsigned char modifyMask[8];

    /* folding level*/
    int level;

    /* base descriptors */
    struct idt_descriptor_t  selfData[MAX_REDIRECT_CHAIN_LENGTH];
    struct idt_descriptor_t *selfPtrs[MAX_REDIRECT_CHAIN_LENGTH];

    /* descriptors chain */
    struct idt_descriptor_t  redirectData[MAX_REDIRECT_CHAIN_LENGTH];
    struct idt_descriptor_t *redirectPtrs[MAX_REDIRECT_CHAIN_LENGTH];

    /* stubs data */
    int_handler_t  handlersData[MAX_REDIRECT_CHAIN_LENGTH];
    int_handler_t *handlersPtrs[MAX_REDIRECT_CHAIN_LENGTH];
} ALIGNED;

/* main idt struct */
struct idt_emulation_t {
    
    /* unload blocked */
    int blocked;
    
    /* subsystem initialized */
    int inited;
    
    /* every idt entry in system described by struct idt_entry_state */
    struct idt_entry_state_t *idtInfo[MAX_PROCESSOR_NUMBER];
} ALIGNED;

#ifdef _HYPER_BINARY_PART_
 /* idt emulation functions */
 void QUICKCALL hypHandleIdtWrite(struct int_handler_stack_t *, int, int, unsigned char *, int);
 void QUICKCALL hypHandleDirtyIdtExecution(struct int_handler_stack_t *);
 int QUICKCALL hypEmulateWriteToIdt(int, int, struct int_handler_stack_t *);
#endif /* _HYPER_BINARY_PART_ */

void QUICKCALL hypIdtEmulInit(void);
int QUICKCALL hypIdtEmulAlloc(void); 
void QUICKCALL hypIdtEmulFree(void);

#endif /* __HYPIDTEMUL_H__ */
