/* ------------------------------------------------------------------- *
 *                                                                     *
 *   Copyright (C) 2005 Parallels Software Inc. All Rights Reserved.   *
 *                      http://www.parallels.com                       *
 *                                                                     *
 * ------------------------------------------------------------------- *
 *                                                                     *
 * Module: hypvmstate.h                                                *
 *                                                                     *
 * Description:                                                        *
 *  Interface beetwen hypervisor and VM.                               *
 *                                                                     *
 * Comments:                                                           *
 *                                                                     *
 * Author: ms                                                          *
 * ------------------------------------------------------------------- *
 */

#ifndef __HYPVMSTATE_H__
#define __HYPVMSTATE_H__

#ifndef _HYPER_BINARY_PART_
# include <linux/mm.h>
# define VMSTATE_SLABNAME "vmStates"
#endif

#include "intgate.h"
//#include "x86.h"

#include "vm_compiler.h"

#define VM_ACTIVATOR_PAGES      3
#define VM_HYPERSWITCH_OFFSET    0

#define VM_HYPERSWITCH_STATE_PAGE_OFFSET (VM_HYPERSWITCH_OFFSET * PAGE_SIZE)

#define VM_IDT_OFFSET           1
#define VM_IDT_STATE_PAGE_OFFSET    (VM_IDT_OFFSET * PAGE_SIZE)

#define VM_JMP_TBL_OFFSET       2
#define VM_JMP_TBL_PAGE_OFFSET      (VM_JMP_TBL_OFFSET * PAGE_SIZE)

struct hyp_vmstate_t {
    /* hs buffer */
    void *stateBuff;

    /* activator physical pages */
    unsigned int vmStatePhyPages[VM_ACTIVATOR_PAGES];
    
    /* vm idt */
    struct idt_descriptor_t *vmIdtd;
    struct idt_info_t vmIdtr;
    
#ifndef _HYPER_BINARY_PART_
    /* pte for handlers */
    pte_entry_t *vmPte;
#else 
    unsigned int unused;
#endif
    
    /* activate function */
    void STDCALL (*vmActivate)(void);
    
    /* id of the state */
    unsigned int stateId;

#ifndef _HYPER_BINARY_PART_
    /* list of vmstates */
    struct list_head list;
#else
    void *unused1[2];
#endif
} ALIGNED;


/////////////////////////////////////////////////////////////////////////////
//
// STRUCTURE: HYPER_SWITCH_BUFF
//
// DESCRIPTION:
//		Buffer for hyper switch state description.
//
///////////////////////////////////////////////////////////////////////////////
typedef struct
{
	//
	// Saved state of primary OS registers
	//

	// GDTR
	char			HyperRetGDT[6];
	// IDTR
	char			HyperRetIDT[6];
	// Return back to primary OS entry point (far pointer)
	char			HyperRetEntry[6];
	// Stack pointer
	char			HyperRetStack[6];
	// LDT selector
	unsigned short	wHyperRetLDTSel;
	// TSS selector
	unsigned short	wHyperRetTSSSel;

	// Saved Control Registers
	unsigned int	uHyperRetCR0;
	unsigned int	uHyperRetCR2;
	unsigned int	uHyperRetCR3;
	unsigned int	uHyperRetCR4;

	// Saved Debug Registers 
	unsigned int	uHyperRetDR0;
	unsigned int	uHyperRetDR1;
	unsigned int	uHyperRetDR2;
	unsigned int	uHyperRetDR3;
	unsigned int	uHyperRetDR6;
	unsigned int	uHyperRetDR7;

	//
	// State of VMM registers
	//

	// GDTR
	char			VmmCurrGDT[6];
	// IDTR
	char			VmmCurrIDT[6];
	// Entry point to VMM segments
	char			VmmCurrSegEntry[6];
	// Stack pointer
	char			VmmCurrStack[6];
	// LDT selector
	unsigned short	wVmmCurrLDTSel;
	// TSS selector
	unsigned short	wVmmCurrTSSSel;
	// VMM FLAT segment base address
	unsigned int	uVmmSegBaseAddr;

	// VMM Control Registers
	unsigned int	uVmmCurrCR0;
	unsigned int	uVmmCurrCR3;
	unsigned int	uVmmCurrCR4;

	// Stack pointer for guest OS start
	unsigned int	uStackESP;


	// Switcher paging CR3 pointer
	unsigned int	uSwitcherCr3;

	// Pointer to the entry point to VMM linear space
	unsigned int	uVmmCurrPagingEntry;
	// Pointer to the exit point from VMM linear space (to the Hypervisor one)
	unsigned int	uVmmCurrPagingExit;

	// Pointer to the HYPER_SWITCH_BUFF structure ...
	// ... on VMM segments and Hypervisor paging
	unsigned int	uVmmSegHyperPagingSwBuff;
	// ... on Hypervisor segments and VMM paging
	unsigned int	uHyperSegVmmPagingSwBuff;


	//
	// System information about switcher code
	//

	// Length of the area (HYPER_SWITCH_BUFF + code)
	unsigned int	uHyperSwitchAreaLen;

	// Offset from buffer begin of RunVM entry point
	unsigned int	uOffs_RunVM;
	// Offset from buffer begin of RetHypervisor entry point
	unsigned int	uOffs_RetHypervisor;

	// Offset from buffer begin of VMM paging entry point
	unsigned int	uOffs_VmmPagingEntry;
	// Offset from buffer begin of VMM segments entry point
	unsigned int	uOffs_VmmSegEntry;
	// Offset from buffer begin of VMM segments exit point
	unsigned int	uOffs_VmmSegExit;
	// Offset from buffer begin of VMM paging exit point
	unsigned int	uOffs_VmmPagingExit;

} ALIGNED HYPER_SWITCH_BUFF;

/* initialization routines for vmStates */
int QUICKCALL hypVmStateInit(void);
void QUICKCALL hypVmStateDeinit(void);

/* vmState allocations/deallocations */
/*
void __attribute__((regparm(0))) *hypVMStateAllocate(void);
int __attribute__((regparm(0))) hypVMStateFree(struct hyp_vmstate_t *);
*/
void STDCALL *hypVMStateAllocate(void);
int STDCALL hypVMStateFree(struct hyp_vmstate_t *);

#endif /* __HYPVMSTATE_H__ */
