/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Parallels Inc. All Rights Reserved.
//		 http://www.parallelssoft.com
//
// Module: ConfigVars.h
//
// Description: Definitions for program
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _CONFIG_VARS_H_
#define _CONFIG_VARS_H_

#if defined(_LINUX_)
# include <asm/page.h>
#endif

// Is we in debug mode ? ( paths to some files point to non /usr/lib directory )

/* don't forget to comment :) */
//#define DEBUG_MODE
//#define LOGGING_ON
#if defined(DEBUG_MODE) || defined(LOGGING_ON)
# warning "Debug/Logging enabled"
#endif

// Define timeout to switch off device activity blinkers ( in milliseconds )
#define BLINKERS_TIMEOUT 	100
// How much recent files is needed to display
#define RECENT_FILES_COUNT	6

#ifdef DEBUG_MODE
	// Comment this to use normal versions of malloc ( can't use in distributed package )
	// #define MALLOC_DEBUG
#endif

// Backtrace stack. Catch segfault
#define BACKTRACE_STACK_SIZE 128

// Use or not Qt default arguments for application command line
#define NOT_USE_REAL_PARAMETERS

// Define font for all application
#define FONT_NAME	"helvetica"
#define FONT_SIZE 	8
#define PIXEL_FONT_SIZE 10

#include "Types.h"

// binary name
#define PVS_BINARY_NAME "/usr/lib/parallels/parallels-linux"

// maximum salt size
#define MAX_SALT_SIZE 1024

// hypervisord pipe
#define HYPERD_PIPE RES_PATH"/hypervisord"

// socket path max
#define UNIX_PATH_MAX 108

// Maximum number of keycodes
#define MAX_KEYCODES_COUNT 		0x1000

// Maximum memory allowed for VM (redefined in libraries into distributed package)
#define VM_MAX_MEM  			1500
// Minimum memory allowed for VM
#define VM_MIN_MEM  			4
// Overall memory limit minimum value
#define VM_OVERALL_MEM_LIMIT_MIN	96
//
//#define VM_GET_OVERALL_MEM_LIMIT(x)	((((x) - (x)/10 - 128) < VM_OVERALL_MEM_LIMIT_MIN) ? VM_OVERALL_MEM_LIMIT_MIN : ((x) - (x)/10 - 128))

#define VM_OVERALL_MEM_LIMIT_OFFSET 128

#ifndef __VM_MODULE__

// QString file name to char* conversion routine
// This conversion is intended to make unicode transitions consistent
// Previous version used QString.toUtf8().data(), but it is not really correct
#define QFLN2CH(name) (QFile::encodeName(name).data())

/* % bases for mem */
#define BASE_512   0.50
#define BASE_1024  0.71
#define BASE_1536  0.61

#define CNT_512    25
#define CNT_1024   52

static inline int getMemLimit(int mem) {

    int cv;
    float i;

    if (mem < 512 && mem > VM_OVERALL_MEM_LIMIT_OFFSET) {
        return (int)(mem * BASE_512);
    } else if (mem > 512 && mem < 1024) {
        for(cv = 512, i = 0; i < 0.21; i += 0.01) {
            cv += CNT_512;
            if (cv >= mem) {
                return (int)(mem * (BASE_512 + i));
            }
        }
    } else if (mem > 1024 && mem < 1536) {
        for(cv = 1024, i = 0; i < 0.1; i += 0.01) {
            cv += CNT_1024;
            if (cv >= mem) {
                return (int)(mem * (BASE_1024 - i));
            }
        }
    } else if (mem > 1536) {
        return (int)(mem * BASE_1536);
    }

    return VM_OVERALL_MEM_LIMIT_MIN;
}

// ८ࠧ ࠧ� �⢮ �࠭��
#define BYTES2PAGES(b)	(((b) + (PAGE_SIZE - 1)) / PAGE_SIZE)

#define VM_GET_OVERALL_MEM_LIMIT(x) (                       \
        (getMemLimit(x) % 4 ) > 0 ?        \
        ((getMemLimit(x) / 4) * 4 ) + 4 :  \
        getMemLimit(x))

#ifndef OVERALL_MEMORY_RESERVE
	#define OVERALL_MEMORY_RESERVE	100
#endif

#ifndef MIN
	#define MIN(x, y) ( ((x)<(y))?(x):(y) )
#endif

#ifndef CALC_RECOMMENDED_MEMORY_LIMIT
/*	#define CALC_RECOMMENDED_MEMORY_LIMIT( uMemForAllVms )							\
			(																		\
				( ( (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE ) < 32 )		\
				? (32)													\
				: (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE				\
			)*/
	/*#define CALC_RECOMMENDED_MEMORY_LIMIT( uMemForAllVms, uMaxPossibleMem )                         \
			MIN(                                                                                                                                            \
				(                                                                                                                                               \
					( ( (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE ) < 32 )         \
					? (32)                                                                                                       \
					: (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE                            \
				)                                                                                                                                               \
				,                                                                                                                                               \
				VM_MAX_MEM                                                                                                                      \
			)*/
#define CALC_RECOMMENDED_MEMORY_LIMIT( uMemForAllVms, uMaxPossibleMem )					\
		MIN(																			\
			(																			\
				( (int)(uMemForAllVms) < 128 )											\
				? (																		\
					( ( (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE ) < 32 )		\
					? (32)																\
					: (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE				\
				  )																		\
				: (																		\
					( ( (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE ) < 64 )		\
					? (64)																\
					: (int)(uMemForAllVms) - (int)OVERALL_MEMORY_RESERVE				\
				  )																		\
			)																			\
			,																			\
			uMaxPossibleMem																\
		)
#endif

// Maximum number of bugreports into 'Bugreports' directory
#define MAX_BUGREPORTS_COUNT 		30

// Define path to all resources files
#ifdef DEBUG_MODE
  #define RES_PATH "/usr/lib/" TARGET_NAME "/"
  //#define RES_PATH "./"
#else 
  #if defined(_LINUX_) || defined(_MACOS_)
    #define RES_PATH "/usr/lib/" TARGET_NAME "/"
  #endif
  #if defined(_BSD_)
    #define RES_PATH "/usr/local/share/" TARGET_NAME "/"
  #endif
#endif

// Path to translation directory
#define TRANSLATION_DIR RES_PATH "resources/"

// Prefix of translation files
#define TRANSFILE_PREFIX "parallels_"

#endif /* __VM_MODULE__ */

// New timer scheme
#define TIMER_RESEARCH

// Do not touch threads priority 
// #define WITHOUT_THREAD_PRIORITY
// Is device threads priority active ?
#define DEVICE_PRIORITY_ACTIVE
// Is main GUI loop priority avtive ?
// #define GUI_PRIORITY_ACTIVE

// Disable SIGALRM generation
#define NO_MILLISEC_TIMER

// Disable timer handler. Time is checking into main loop.
#define TIMER_IN_LOOP

#ifdef DEBUG_MODE
  // Log timing of pthread operations
  // #define TIMING_OUT
#endif

// Call monitor passthru interrupt gate
#define CALL_MONITOR_FROM_GATE

// Sliding mouse enabled
#define SLIDING_MOUSE_SUPPORT

#define DYNAMIC_PHYSICAL_MEMORY
#ifdef DYNAMIC_PHYSICAL_MEMORY
	// #define CHECK_WITH_CRC
	#define MAX_MEMORY_BLOCK	1024
	
	// memory block size is a configuration parameter
	#define BLOCK_SIZE_AS_PARAMETER

	#define MAX_ADJUST_WS_PAGES		1024
	#define MAX_ADJUST_WS			MAX_ADJUST_WS_PAGES / MAX_MEMORY_BLOCK
	#define MAX_PACKED_MEMORY_BLOCKS	MAX_ADJUST_WS
#endif	

#define FAST_STUBS

#ifdef FAST_STUBS
	// Allow fast stubs JMP  CALL
	#define FAST_STUBS_BY_JMP
#endif

// Following section was added by VitalB on 2005-10-12
#define USE_VIDEO_MODE_1152_864	1

// if LPT_RFIX_UCMDB is not defined then async. Lpt is not used
#define LPT_RFIX_UCMDB

// Maximum allowable host physical memory size supported by our PAE
#define	PAE_MAX_MEM_MB	(4 * 1024)

// Maximum allowable host physiscal memory size supported by our PAE with reserve
#define PAE_MAX_MEM_RESERVE_MB (3 * 1024)

#define SUPPORT_SAVE_MONITOR_STATE

// Define size by videomodes
#define VBE_LFB_SIZE_B	(1600 * 1200 * 3 + 256 * 4)
// Define size of buffer through videomodes
#define VBE_LFB_SIZE_MB_VMODE ((VBE_LFB_SIZE_B + 1024*1024 - 1) >> 20) 
// Define size of buffer directly
#define VBE_LFB_SIZE_MB			8
// Decide what to use
#if VBE_LFB_SIZE_MB_VMODE < VBE_LFB_SIZE_MB
   #define VBE_LFB_SIZE_MB_GUESSED VBE_LFB_SIZE_MB
#else
   #define VBE_LFB_SIZE_MB_GUESSED VBE_LFB_SIZE_MB_VMODE
#endif

#define VBE_LFB_SIZE	( VBE_LFB_SIZE_MB_GUESSED << 20)

#define VBE_LFB_SIZE_APPLICATION (VBE_LFB_SIZE * 2)

// Size of vesa buffer in pages
#define VBE_LFB_PAGES	BYTES2PAGES(VBE_LFB_SIZE)

// When enabled, VTx is always switched on and off every hyper switch
// Otherwise, VTx is enabled one time by hypervisor
#define VTX_ALWAYS_ON_OFF

#define WRITE_TO_PAGE_DIR_TBL_WITH_RESET
#ifndef WRITE_TO_PAGE_DIR_TBL_WITH_RESET
        // if ON  - disable reset of PageDirs & PageTables by OsType else
        // if OFF - disable reset if guest Os uses Invlpg instruction
        #define DISABLE_RESET_BY_OSTYPE
#endif

//#define _HYP_IDTR_CHECK_

#endif
