#!/bin/sh
#############################################################
# Copyright (C) 2005 Parallels Inc. All Rights Reserved.
# Install application
#############################################################

. ./data/gettext.sh

srcdir=`dirname $0` && cd $srcdir
localedir='/usr/share/locale'

export TEXTDOMAIN=parallels
export TEXTDOMAINDIR=./data/resources/locale

v_maj="`grep 'VER_MAJOR' ./data/parallels-version | awk '{print $2;}'`"
v_min="`grep 'VER_MINOR' ./data/parallels-version | awk '{print $2;}'`"
v_build="`grep 'VER_BUILD' ./data/parallels-version | awk '{print $2;}'`"

mainname="Parallels"
submname="Workstation"
mainver="$v_maj.$v_min"

## check for root
if test `id -u` -ne 0; then
    gettext "Error: you must be root to run this script"; echo
    exit
fi

## Check for running programs
if test -e "/usr/lib/parallels/autostart/drivers_stop"; then
    /usr/lib/parallels/autostart/drivers_stop > /dev/null 2>&1
fi

if test "x`ps -A | grep -iw parallels | grep -v -`" != "x"; then
    eval_gettext "Error: Found another running instance of \$mainname \$submname
Error: please close it before installing"; echo
    exit -2
fi
if test "x`ps -A | grep -iw imagetool`" != "x"; then
    gettext "Error: Found another running instance of Image Tool
Error: please close it before installing"; echo
    exit -2
fi
if test "x`ps -A | grep -iw updatewizard`" != "x"; then
    gettext "Error: Found another running instance of Update Wizard
Error: please close it before installing"; echo
    exit -2
fi

v_os="LINUX"
v_ipath=./data/
v_path=/usr/lib/parallels
resdir="$v_path/resources"

echo
eval_gettext "Installing \$mainname \$submname \$mainver build \$v_build"; echo
echo

is_new=1
if [ -f /usr/bin/Parallels ]; then
    output=`stat -c "%N" /usr/bin/Parallels | grep -e '->'`
    if [ -z "$output" ]; then
        is_new=0
    else
        echo $output | grep -q /usr/lib/Parallels/Parallels-LINUX; err=$?
        if [ $err -eq 0 ]; then is_new=0; fi
    fi
fi

if [ -f $v_path/uninstall.sh ]; then
    $v_path/uninstall.sh
else
    ./uninstall.sh
fi

# Create dirs for install. 
mkdir $v_path > /dev/null 2>&1
chmod a+rx $v_path

gettext "     Copying files..."; echo
cp -rf ./data/. $v_path

if test ! -d /usr/share/doc/parallels; then 
    mkdir /usr/share/doc/parallels
    chmod a+rx /usr/share/doc/parallels
fi
cp -rf ./data/doc/. /usr/share/doc/parallels/

# Set rights to directories
find $v_path -type d -name '*' | while read file; do
    chmod 755 $file
done

# Set rights to tools image
chmod 644 $v_path/tools/vmtools.fdd
chmod 644 $v_path/tools/vmtools.iso

# And create bugreport directory
mkdir $v_path/bugreports > /dev/null 2>&1
chmod 1777 $v_path/bugreports  > /dev/null 2>&1

# Create .Parallels_common_options
touch $v_path/.parallels_common_options > /dev/null 2>&1
chmod 666 $v_path/.parallels_common_options > /dev/null 2>&1

# Set permissions to main path
chmod 755 $v_path

# workaround for users forgotten to pass '-p' to tar
chmod 6555 $v_path/parallels-linux

mv $v_path/parallels /usr/bin/parallels

# Setup links
ln -sf $v_path/parallels-config /usr/bin/parallels-config
ln -sf $v_path/imagetool /usr/bin/
ln -sf $v_path/updatewizard /usr/bin/
ln -sf /usr/bin/parallels-config /usr/bin/Parallels-config
ln -sf /usr/bin/parallels /usr/bin/Parallels
ln -sf /usr/bin/imagetool /usr/bin/ImageTool

rm -f $v_path/.ereaded > /dev/null 1>&1
touch $v_path/.not_configured

prelink="/etc/prelink.conf"
if [ -e "$prelink" ]; then
	if [ `grep -c /usr/lib/parallels $prelink` -eq 0 ]; then
		echo "-b /usr/lib/parallels" >> $prelink
	fi
fi

for langdir in `ls -d $resdir/locale/* 2> /dev/null`; do
    lang=`echo $langdir | sed -re 's/^.*\/([^\/]*)$/\1/'`
    lfile="$langdir/LC_MESSAGES/parallels"

    if [ -d "$localedir/$lang" ]; then
        ln -sf $lfile.mo "$localedir/$lang/LC_MESSAGES/parallels.mo"
    fi
done

if [ $is_new -eq 0 ]; then
    olddir="/usr/lib/Parallels"
    cp -f $olddir/.Parallels_common_options $v_path/.parallels_common_options 2> /dev/null
    cp -f $olddir/.Parallels_license $v_path/.parallels_license 2> /dev/null
    cp -f $olddir/.DHCPd_configuration $v_path/.dhcpd_configuration 2> /dev/null
    cp -f $olddir/.dhcp.* $v_path/ 2> /dev/null
    cp -a $olddir/Bugreports/. $v_path/bugreports/ 2> /dev/null

fi

# even if you reach this string, it does not mean that you can run it.
eval_gettext "Installation has been completed.

      You should configure \$mainname \$submname \$mainver
      before starting it for the first time.
      Issue \"parallels-config\" command.
"; echo
