#!/bin/sh
#############################################################
# Copyright (C) 2005 Parallels Inc. All Rights Reserved.
# Uninstall application
#############################################################

v_os="LINUX"
v_path=/usr/lib/parallels
old_path=/usr/lib/Parallels

if [ `dirname $0` == "$v_path" ]; then
    . $v_path/gettext.sh
else
    . ./data/gettext.sh
    export TEXTDOMAINDIR=./data/resources/locale
fi

export TEXTDOMAIN=parallels

v_maj="`grep 'VER_MAJOR' ./data/parallels-version | awk '{print $2;}'`"
v_min="`grep 'VER_MINOR' ./data/parallels-version | awk '{print $2;}'`"

mainname="Parallels"
submname="Workstation"
mainver="$v_maj.$v_min"

## check for root
if test `id -u` -ne 0; then
    gettext "Error: you must be root to run this script"; echo
    exit
fi

if test "x`ps -A | grep -iw parallels`" != "x"; then
    eval_gettext "Error: Found another running instance of \$mainname \$submname
Error: please close it before uninstalling"; echo
    exit -2
fi
if test "x`ps -A | grep -iw imagetool`" != "x"; then
    gettext "Error: Found another running instance of Image Tool
Error: please close it before uninstalling"; echo
    exit -2
fi

is_new=1
if [ -f /usr/bin/Parallels ]; then
    output=`stat -c "%N" /usr/bin/Parallels | grep -e '->'`
    if [ -z "$output" ]; then
        is_new=0
    else
        echo $output | grep -q /usr/lib/Parallels/Parallels-LINUX; err=$?
        if [ $err -eq 0 ]; then is_new=0; fi
    fi
fi

# Check existance base application directory
if test ! -d $v_path; then exit; fi

echo
eval_gettext "Uninstalling \$mainname \$submname \$mainver"; echo
echo

gettext "     Uninstalling drivers..."; echo

if [ $is_new -ne 0 ]; then
    test -f "$v_path/autostart/drivers_stop" && $v_path/autostart/drivers_stop
    test -f "$v_path/autostart/iscripts" && $v_path/autostart/iscripts remove > /dev/null 2>&1
    test -f "$v_path/tools/mimelink" && $v_path/tools/mimelink dissociate
else
    test -f "$old_path/Autostart/DriversStop" && $old_path/Autostart/DriversStop
    test -f "$old_path/Autostart/iscripts" && $old_path/Autostart/iscripts remove > /dev/null 2>&1
    test -f "$old_path/Tools/mimelink" && $old_path/Tools/mimelink dissociate
fi

rm -rf /dev/vm-main /dev/vm-bridge /dev/hypervisor

gettext "     Deleting files..."; echo

rm -f  /usr/bin/[pP]arallels
rm -f  /usr/bin/[pP]arallels-config
rm -f  /usr/bin/[Ii]mage[Tt]ool
rm -f  /usr/bin/[Uu]pdate[Ww]izard
rm -rf /usr/share/doc/[pP]arallels

if [ $is_new -ne 0 ]; then
    rm -rf $v_path/[^b]*
    rm -f  $v_path/.ereaded > /dev/null 1>&1
else
    rm -rf $old_path/[^B]*
    rm -f  $old_path/.ereaded > /dev/null 1>&1
fi

find /usr/share/locale/ -name parallels.mo -exec rm -f {} \;

if [ -e /etc/prelink.conf ]; then
    sed -e '/parallels/d' /etc/prelink.conf | sed -e '/Parallels/d' > /etc/prelink.conf.new && mv -f /etc/prelink.conf.new /etc/prelink.conf
fi

gettext "Uninstallation has been completed."; echo
